'''Cognite API SDK

This package has been created to ensure excellent excellent user experience for data scientists using the Cognite
Data Platform (CDP).
'''

__all__ = ['assets', 'config', 'similarity_search', 'tagmatching', 'timeseries']
